<?php
/**
 * Plugin Name: favicon picker
 * Plugin URI: http://clorith.net
 * Description: Add a favicon to your site, by inserting a rewrite for modern browsers, as well as header tags!
 * Version: 1.0
 * Author: Clorith
 *
 */

    class mrstk_favico {

        /*
         * Update our rewrite rule if the favicon setting is changed
         *
         * @param string $option Name of the option being updated
         * @param mixed @oldvalue The old settings
         * @param mixed @_newvalue The new settings values containing our favicon URL
         */
        function rewrite_update( $option, $oldvalue, $_newvalue ) {
            if ( $option == 'mrstk_favico_options' )
            {
                add_rewrite_rule(
                    'favicon.ico$',
                    str_replace( get_bloginfo( 'wpurl' ), '', $_newvalue['address'] ),
                    'top'
                );

                flush_rewrite_rules();
            }
        }

        /*
         * Add a rewrite URL when a favicon is first set (first time use)
         *
         * @param string $option Name of the option being updated
         * @param mixed @_value The settings values containing our favicon URL
         */
        function rewrite_add( $option, $_value ) {
            if ( $option == 'mrstk_favico_options' )
            {
                add_rewrite_rule(
                    'favicon.ico$',
                    str_replace( get_bloginfo( 'wpurl' ), '', $_value['address'] ),
                    'top'
                );

                flush_rewrite_rules();
            }
        }

        /*
         * Add the proposed favicon relation link to the header, this is not guaranteed to always show the favicon, and browsers will still hit WPURL.TLD/favicon.ico by default, hence our rewrites
         */
        function wp_head() {
            $options = get_option( 'mrstk_favico_options' );

            echo '
                <link rel="shortcut icon" href="' . $options['address'] . '" />
            ';
        }

        /*
         * Add our plugin page to the menu under Settings
         */
        function menu() {
            add_options_page( 'favicon picker', 'favicon.ico', 'manage_options', 'mrstk-favico', array( &$this, 'options' ) );
        }

        /*
         * Callback for our menu function, used to display the options page
         *
         * Kudos to Ram Ratan Maurya at codestag, where I picked up the javascript bit for implementing the WordPress media uploader!
         */
        function options() {
            ?>
                <div class="wrap">
                    <h2>favicon.ico</h2>

                    <form action="options.php" method="post">
                        <?php settings_fields( 'mrstk_favico_options' ) ?>
                        <?php do_settings_sections( 'mrstk_favico' ) ?>

                        <?php submit_button() ?>
                    </form>
                </div>

                <script type="text/javascript">
                    jQuery(document).ready(function($){

                        var _custom_media = true,
                                _orig_send_attachment = wp.media.editor.send.attachment;

                        $('#mrstk_favico_address_button').click(function(e) {
                            var send_attachment_bkp = wp.media.editor.send.attachment;
                            var button = $(this);
                            var id = button.attr('id').replace('_button', '');
                            _custom_media = true;
                            wp.media.editor.send.attachment = function(props, attachment){
                                if ( _custom_media ) {
                                    $("#" + id).val(attachment.url);
                                } else {
                                    return _orig_send_attachment.apply( this, [props, attachment] );
                                }
                            };
                            wp.media.editor.open(button);
                            return false;
                        });
                        $('.add_media').on('click', function(){
                            _custom_media = false;
                        });
                    });
                </script>
            <?php
        }

        /*
         * Initiate our plugin when the admin page is loaded
         */
        function admin_init() {
            if ( $_GET['page'] == 'mrstk-favico' )
            {
                wp_enqueue_media();
                register_setting( 'mrstk_favico_options', 'mrstk_favico_options' );
                add_settings_section( 'mrstk_favico_main', 'Main settings', array( &$this, 'helper_text' ), 'mrstk_favico' );
                add_settings_field( 'mrstk_favico_address', 'favicon.ico', array( &$this, 'settings' ), 'mrstk_favico', 'mrstk_favico_main' );
            }
        }

        function helper_text() {
            echo '
                <p>Supply the link to your fav.ico file, or you may use the media browser to upload it on your own.</p>
            ';
        }

        /*
         * Insert settings to our page, we only need this one bit as we only really have one setting
         */
        function settings() {
            $options = get_option( 'mrstk_favico_options' );
            echo '
                <input id="mrstk_favico_address" name="mrstk_favico_options[address]" size="40" type="text" value="' . $options['address'] . '" />
                <input class="button" name="mrstk_favico_address_button" id="mrstk_favico_address_button" value="Upload" />
            ';
        }

    }

    // Create our class for referencing functions
    $mrstk_favico = new mrstk_favico();

    /*
     * Add WordPress hooks for various things we need to do
     */
    add_action( 'admin_init', array( &$mrstk_favico, 'admin_init' ) );
    add_action( 'admin_menu', array( &$mrstk_favico, 'menu' ) );
    add_action( 'updated_option', array( &$mrstk_favico, 'rewrite_update' ), 10, 3 );
    add_action( 'added_option', array( &$mrstk_favico, 'rewrite_add' ), 10, 2 );
    add_action( 'wp_head', array( &$mrstk_favico, 'wp_head' ) );